@@ -18,7 +18,13 @@ class TestGemDocManager < RubyGemTestCase
 
   def test_uninstall_doc_unwritable
     orig_mode = File.stat(@spec.installation_path).mode
-    File.chmod 0, @spec.installation_path
+    
+    # File.chmod has no effect on MS Windows directories (it needs ACL).
+    if win_platform?
+      skip("test_uninstall_doc_unwritable skipped on MS Windows")
+    else
+      File.chmod(0, @spec.installation_path)
+    end
 
     assert_raises Gem::FilePermissionError do
       @manager.uninstall_doc
