@@ -1,15 +1,14 @@
-#!/usr/bin/env ruby
 #--
 # Copyright 2006 by Chad Fowler, Rich Kilmer, Jim Weirich and others.
 # All rights reserved.
 # See LICENSE.txt for permissions.
 #++
 
-require 'test/unit'
 require File.join(File.expand_path(File.dirname(__FILE__)), 'gemutilities')
 require 'webrick'
 require 'zlib'
 require 'rubygems/remote_fetcher'
+require 'ostruct'
 
 # = Testing Proxy Settings
 #
@@ -24,7 +23,7 @@ require 'rubygems/remote_fetcher'
 # Note that the proxy server is not a *real* proxy server.  But our
 # software doesn't really care, as long as we hit the proxy URL when a
 # proxy is configured.
-#
+
 class TestGemRemoteFetcher < RubyGemTestCase
 
   include Gem::DefaultUserInteraction
@@ -105,20 +104,30 @@ gems:
 
     @a1, @a1_gem = util_gem 'a', '1' do |s| s.executables << 'a_bin' end
 
-    Gem::RemoteFetcher.instance_variable_set :@fetcher, nil
+    Gem::RemoteFetcher.fetcher = nil
+
+    @fetcher = Gem::RemoteFetcher.fetcher
+  end
+
+  def teardown
+    super
+    Gem.configuration[:http_proxy] = nil
   end
 
   def test_self_fetcher
     fetcher = Gem::RemoteFetcher.fetcher
-    assert_not_nil fetcher
+    refute_nil fetcher
     assert_kind_of Gem::RemoteFetcher, fetcher
   end
 
   def test_self_fetcher_with_proxy
     proxy_uri = 'http://proxy.example.com'
     Gem.configuration[:http_proxy] = proxy_uri
+    Gem::RemoteFetcher.fetcher = nil
+
     fetcher = Gem::RemoteFetcher.fetcher
-    assert_not_nil fetcher
+
+    refute_nil fetcher
     assert_kind_of Gem::RemoteFetcher, fetcher
     assert_equal proxy_uri, fetcher.instance_variable_get(:@proxy_uri).to_s
   end
@@ -126,8 +135,11 @@ gems:
   def test_self_fetcher_with_proxy_URI
     proxy_uri = URI.parse 'http://proxy.example.com'
     Gem.configuration[:http_proxy] = proxy_uri
+    Gem::RemoteFetcher.fetcher = nil
+
     fetcher = Gem::RemoteFetcher.fetcher
-    assert_not_nil fetcher
+    refute_nil fetcher
+
     assert_kind_of Gem::RemoteFetcher, fetcher
     assert_equal proxy_uri, fetcher.instance_variable_get(:@proxy_uri)
   end
@@ -135,32 +147,31 @@ gems:
   def test_fetch_size_bad_uri
     fetcher = Gem::RemoteFetcher.new nil
 
-    e = assert_raise ArgumentError do
+    e = assert_raises ArgumentError do
       fetcher.fetch_size 'gems.example.com/yaml'
     end
 
-    assert_equal 'uri is not an HTTP URI', e.message
+    assert_equal 'uri scheme is invalid', e.message
   end
 
   def test_fetch_size_socket_error
     fetcher = Gem::RemoteFetcher.new nil
-    def fetcher.connect_to(host, port)
-      raise SocketError
+    def fetcher.connection_for(uri)
+      raise SocketError, "tarded"
     end
 
     uri = 'http://gems.example.com/yaml'
-    e = assert_raise Gem::RemoteFetcher::FetchError do
+    e = assert_raises Gem::RemoteFetcher::FetchError do
       fetcher.fetch_size uri
     end
 
-    assert_equal "SocketError (SocketError)\n\tgetting size of #{uri}", e.message
+    assert_equal "SocketError: tarded (#{uri})", e.message
   end
 
   def test_no_proxy
     use_ui @ui do
-      fetcher = Gem::RemoteFetcher.new nil
-      assert_data_from_server fetcher.fetch_path(@server_uri)
-      assert_equal SERVER_DATA.size, fetcher.fetch_size(@server_uri)
+      assert_data_from_server @fetcher.fetch_path(@server_uri)
+      assert_equal SERVER_DATA.size, @fetcher.fetch_size(@server_uri)
     end
   end
 
@@ -182,7 +193,7 @@ gems:
           @test_data
         end
 
-        raise Gem::RemoteFetcher::FetchError, "haha!"
+        raise Gem::RemoteFetcher::FetchError.new("haha!", nil)
       end
     end
 
@@ -226,6 +237,21 @@ gems:
                  inst.download(@a1, local_path)
   end
 
+  def test_download_local_space
+    space_path = File.join @tempdir, 'space path'
+    FileUtils.mkdir space_path
+    FileUtils.mv @a1_gem, space_path
+    local_path = File.join space_path, "#{@a1.full_name}.gem"
+    inst = nil
+
+    Dir.chdir @tempdir do
+      inst = Gem::RemoteFetcher.fetcher
+    end
+
+    assert_equal File.join(@gemhome, 'cache', "#{@a1.full_name}.gem"),
+                 inst.download(@a1, local_path)
+  end
+
   def test_download_install_dir
     a1_data = nil
     File.open @a1_gem, 'rb' do |fp|
@@ -237,6 +263,7 @@ gems:
     install_dir = File.join @tempdir, 'more_gems'
 
     a1_cache_gem = File.join install_dir, 'cache', "#{@a1.full_name}.gem"
+    FileUtils.mkdir_p(File.dirname(a1_cache_gem))
     actual = fetcher.download(@a1, 'http://gems.example.com', install_dir)
 
     assert_equal a1_cache_gem, actual
@@ -246,7 +273,7 @@ gems:
     assert File.exist?(a1_cache_gem)
   end
 
-  unless win_platform? then # File.chmod doesn't work
+  unless win_platform? # File.chmod doesn't work
     def test_download_local_read_only
       FileUtils.mv @a1_gem, @tempdir
       local_path = File.join @tempdir, "#{@a1.full_name}.gem"
@@ -262,6 +289,19 @@ gems:
     ensure
       File.chmod 0755, File.join(@gemhome, 'cache')
     end
+
+    def test_download_read_only
+      File.chmod 0555, File.join(@gemhome, 'cache')
+      File.chmod 0555, File.join(@gemhome)
+
+      fetcher = util_fuck_with_fetcher File.read(@a1_gem)
+      fetcher.download(@a1, 'http://gems.example.com')
+      assert File.exist?(File.join(Gem.user_dir, 'cache',
+                                   "#{@a1.full_name}.gem"))
+    ensure
+      File.chmod 0755, File.join(@gemhome)
+      File.chmod 0755, File.join(@gemhome, 'cache')
+    end
   end
 
   def test_download_platform_legacy
@@ -291,7 +331,7 @@ gems:
   def test_download_unsupported
     inst = Gem::RemoteFetcher.fetcher
 
-    e = assert_raise Gem::InstallError do
+    e = assert_raises Gem::InstallError do
       inst.download @a1, 'ftp://gems.rubyforge.org'
     end
 
@@ -362,43 +402,87 @@ gems:
     end
   end
 
+  def test_fetch_path_gzip
+    fetcher = Gem::RemoteFetcher.new nil
+
+    def fetcher.open_uri_or_path(uri, mtime, head = nil)
+      Gem.gzip 'foo'
+    end
+
+    assert_equal 'foo', fetcher.fetch_path(@uri + 'foo.gz')
+  end
+
+  def test_fetch_path_gzip_unmodified
+    fetcher = Gem::RemoteFetcher.new nil
+
+    def fetcher.open_uri_or_path(uri, mtime, head = nil)
+      nil
+    end
+
+    assert_equal nil, fetcher.fetch_path(@uri + 'foo.gz', Time.at(0))
+  end
+
   def test_fetch_path_io_error
     fetcher = Gem::RemoteFetcher.new nil
 
-    def fetcher.open_uri_or_path(uri) raise EOFError; end
+    def fetcher.open_uri_or_path(uri, mtime, head = nil)
+      raise EOFError
+    end
 
-    e = assert_raise Gem::RemoteFetcher::FetchError do
+    e = assert_raises Gem::RemoteFetcher::FetchError do
       fetcher.fetch_path 'uri'
     end
 
-    assert_equal 'EOFError: EOFError reading uri', e.message
+    assert_equal 'EOFError: EOFError (uri)', e.message
+    assert_equal 'uri', e.uri
   end
 
   def test_fetch_path_socket_error
     fetcher = Gem::RemoteFetcher.new nil
 
-    def fetcher.open_uri_or_path(uri) raise SocketError; end
+    def fetcher.open_uri_or_path(uri, mtime, head = nil)
+      raise SocketError
+    end
 
-    e = assert_raise Gem::RemoteFetcher::FetchError do
+    e = assert_raises Gem::RemoteFetcher::FetchError do
       fetcher.fetch_path 'uri'
     end
 
-    assert_equal 'SocketError: SocketError reading uri', e.message
+    assert_equal 'SocketError: SocketError (uri)', e.message
+    assert_equal 'uri', e.uri
   end
 
   def test_fetch_path_system_call_error
     fetcher = Gem::RemoteFetcher.new nil
 
-    def fetcher.open_uri_or_path(uri);
+    def fetcher.open_uri_or_path(uri, mtime = nil, head = nil)
       raise Errno::ECONNREFUSED, 'connect(2)'
     end
 
-    e = assert_raise Gem::RemoteFetcher::FetchError do
+    e = assert_raises Gem::RemoteFetcher::FetchError do
       fetcher.fetch_path 'uri'
     end
 
-    assert_match %r|ECONNREFUSED:.*connect\(2\) reading uri\z|,
+    assert_match %r|ECONNREFUSED:.*connect\(2\) \(uri\)\z|,
                  e.message
+    assert_equal 'uri', e.uri
+  end
+
+  def test_fetch_path_unmodified
+    fetcher = Gem::RemoteFetcher.new nil
+
+    def fetcher.open_uri_or_path(uri, mtime, head = nil)
+      nil
+    end
+
+    assert_equal nil, fetcher.fetch_path(URI.parse(@gem_repo), Time.at(0))
+  end
+
+  def test_get_proxy_from_env_auto_normalizes
+    fetcher = Gem::RemoteFetcher.new(nil)
+    ENV['HTTP_PROXY'] = 'fakeurl:12345'
+
+    assert_equal('http://fakeurl:12345', fetcher.get_proxy_from_env.to_s)
   end
 
   def test_get_proxy_from_env_empty
@@ -458,7 +542,7 @@ gems:
     def conn.request(req)
       unless defined? @requested then
         @requested = true
-        res = Net::HTTPRedirection.new nil, 301, nil
+        res = Net::HTTPMovedPermanently.new nil, 301, nil
         res.add_field 'Location', 'http://gems.example.com/real_path'
         res
       else
@@ -471,9 +555,9 @@ gems:
     conn = { 'gems.example.com:80' => conn }
     fetcher.instance_variable_set :@connections, conn
 
-    fetcher.send :open_uri_or_path, 'http://gems.example.com/redirect' do |io|
-      assert_equal 'real_path', io.read
-    end
+    data = fetcher.open_uri_or_path 'http://gems.example.com/redirect'
+
+    assert_equal 'real_path', data
   end
 
   def test_open_uri_or_path_limited_redirects
@@ -482,7 +566,7 @@ gems:
     conn = Object.new
     def conn.started?() true end
     def conn.request(req)
-      res = Net::HTTPRedirection.new nil, 301, nil
+      res = Net::HTTPMovedPermanently.new nil, 301, nil
       res.add_field 'Location', 'http://gems.example.com/redirect'
       res
     end
@@ -490,29 +574,44 @@ gems:
     conn = { 'gems.example.com:80' => conn }
     fetcher.instance_variable_set :@connections, conn
 
-    e = assert_raise Gem::RemoteFetcher::FetchError do
-      fetcher.send :open_uri_or_path, 'http://gems.example.com/redirect'
+    e = assert_raises Gem::RemoteFetcher::FetchError do
+      fetcher.open_uri_or_path 'http://gems.example.com/redirect'
     end
 
-    assert_equal 'too many redirects', e.message
+    assert_equal 'too many redirects (http://gems.example.com/redirect)',
+                 e.message
   end
 
-  def test_zip
-    use_ui @ui do
-      self.class.enable_zip = true
-      fetcher = Gem::RemoteFetcher.new nil
-      assert_equal SERVER_DATA.size, fetcher.fetch_size(@server_uri), "probably not from proxy"
-      zip_data = fetcher.fetch_path(@server_z_uri)
-      assert zip_data.size < SERVER_DATA.size, "Zipped data should be smaller"
-    end
+  def test_request
+    uri = URI.parse "#{@gem_repo}/specs.#{Gem.marshal_version}"
+    util_stub_connection_for :body => :junk, :code => 200
+
+    response = @fetcher.request uri, Net::HTTP::Get
+
+    assert_equal 200, response.code
+    assert_equal :junk, response.body
   end
 
-  def test_no_zip
-    use_ui @ui do
-      self.class.enable_zip = false
-      fetcher = Gem::RemoteFetcher.new nil
-      assert_error { fetcher.fetch_path(@server_z_uri) }
-    end
+  def test_request_head
+    uri = URI.parse "#{@gem_repo}/specs.#{Gem.marshal_version}"
+    util_stub_connection_for :body => '', :code => 200
+    response = @fetcher.request uri, Net::HTTP::Head
+
+    assert_equal 200, response.code
+    assert_equal '', response.body
+  end
+
+  def test_request_unmodifed
+    uri = URI.parse "#{@gem_repo}/specs.#{Gem.marshal_version}"
+    conn = util_stub_connection_for :body => '', :code => 304
+
+    t = Time.now
+    response = @fetcher.request uri, Net::HTTP::Head, t
+
+    assert_equal 304, response.code
+    assert_equal '', response.body
+
+    assert_equal t.rfc2822, conn.payload['if-modified-since']
   end
 
   def test_yaml_error_on_size
@@ -523,7 +622,17 @@ gems:
     end
   end
 
-  private
+  def util_stub_connection_for hash
+    def @fetcher.connection= conn
+      @conn = conn
+    end
+
+    def @fetcher.connection_for uri
+      @conn
+    end
+
+    @fetcher.connection = Conn.new OpenStruct.new(hash)
+  end
 
   def assert_error(exception_class=Exception)
     got_exception = false
@@ -543,6 +652,20 @@ gems:
     assert_block("Data is not from proxy") { data =~ /0\.4\.2/ }
   end
 
+  class Conn
+    attr_accessor :payload
+
+    def initialize(response)
+      @response = response
+      self.payload = nil
+    end
+
+    def request(req)
+      self.payload = req
+      @response
+    end
+  end
+
   class NilLog < WEBrick::Log
     def log(level, data) #Do nothing
     end
